# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2018, 2019. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************
# Load the data
import numpy as np
from scipy.sparse import load_npz
import time
import sys
import argparse
import pandas as pd
from sklearn.metrics import classification_report, roc_curve, auc, precision_recall_curve
from sklearn.preprocessing import StandardScaler
from sklearn.utils import compute_class_weight
from sklearn.model_selection import StratifiedShuffleSplit
from sklearn.preprocessing import normalize

defaultPath = "."

CLI=argparse.ArgumentParser()
CLI.add_argument(
   "--data_path",
   type=str,
   default=defaultPath
)
CLI.add_argument(
   "--device_ids",
   nargs="*",
   type=int,
   default=[]
)
CLI.add_argument(
    "--use_gpu",
    action = 'store_true',
    default=False
)
CLI.add_argument(
    "--skip_sklearn",
    action = 'store_true',
    default=False
)
CLI.add_argument(
   "--use_balanced_class_weights",
   action = 'store_true',
   default=False
)
CLI.add_argument(
   "--fit_intercept",
   action = 'store_true',
   default=False
)
CLI.add_argument(
   "--return_training_history",
   type=str,
   default=None
)
args = CLI.parse_args()

t0 = time.time()
# Import the data
X_train = np.load(args.data_path + "/data/creditcard.X_train.npy")
X_test  = np.load(args.data_path + "/data/creditcard.X_test.npy")
y_train = np.load(args.data_path + "/data/creditcard.y_train.npy")
y_test  = np.load(args.data_path + "/data/creditcard.y_test.npy")
print("Data load time (s): {0:.2f}".format(time.time()-t0))

if args.use_gpu:
  num_threads = 256
  cpu_gpu = "GPU"
else:
  num_threads = 8
  cpu_gpu = "CPU"

# num_threads for predict probability/ predict methods
p_num_threads = 2
 
if args.use_balanced_class_weights:
  class_weight = "balanced"
else:
  class_weight = None

# Import the LogisticRegression from pai4sk
from pai4sk import LogisticRegression
lr = LogisticRegression(use_gpu=args.use_gpu, device_ids=args.device_ids,
                        num_threads=num_threads, class_weight=class_weight,
                        fit_intercept=args.fit_intercept, regularizer=100,
                        return_training_history=args.return_training_history)

# Training
t0 = time.time()
lr.fit(X_train, y_train)
print("[pai4sk] Training time (s):  {1:.2f}".format(cpu_gpu, time.time()-t0))

# Evaluate log-loss on test set
pred = lr.predict_proba(X_test)[:,1]

from sklearn.metrics import average_precision_score
acc_snap = average_precision_score(y_test, pred)
print("[pai4sk] Average Precision Score :   {1:.4f}".format(cpu_gpu, acc_snap))

# Import the LogisticRegression from pai4sk.linear_model 
from pai4sk.linear_model import LogisticRegression
lr = LogisticRegression(use_gpu=args.use_gpu, device_ids=args.device_ids,
                        num_threads=num_threads, class_weight=class_weight,
                        fit_intercept=args.fit_intercept, dual=True, tol=0.001, C=1.0/100,
                        return_training_history=args.return_training_history)

# Training time
t0 = time.time()
lr.fit(X_train, y_train)
print("[pai4sk.lmodel] Training time (s):  {0:.2f}".format(time.time()-t0))

# Inference
pred = lr.predict_proba(X_test)[:,1]

from sklearn.metrics import average_precision_score
acc_snap = average_precision_score(y_test, pred)
print("[pai4sk] Average Precision Score :   {1:.4f}".format(cpu_gpu, acc_snap))

if args.skip_sklearn:
    print('Execution using scikit-learn is skipped.')
    exit()

# Import sklearn's LogisticRegression from pai4sk module directly 
from pai4sk.linear_model import LogisticRegressionSklearn as LogisticRegression
lr = LogisticRegression(fit_intercept=False, dual=True, tol=0.001, 
                        class_weight=class_weight, random_state=42, C=1.0/100)

# Training time
t0 = time.time()
lr.fit(X_train, y_train)
print("[sklearn] Training time (s):  {0:.2f}".format(time.time()-t0))

pred = lr.predict_proba(X_test)[:,1]

from sklearn.metrics import average_precision_score
acc_snap = average_precision_score(y_test, pred)
print("[sklearn] Average Precision Score :   {1:.4f}".format(cpu_gpu, acc_snap))
