# Criteo-Kaggle Example

In this example we will train a Logistic Regression model on the criteo-kaggle dataset, using both `scikit-learn` and `snap-ml-local`. 

The criteo-kaggle dataset is from the [Display Advertising Challenge](https://www.kaggle.com/c/criteo-display-ad-challenge).


## Download the Data

Download and decompress the data from the LIBSVM repository:
```
mkdir data
cd data
wget https://s3-us-west-2.amazonaws.com/criteo-public-svm-data/criteo.kaggle2014.svm.tar.gz
tar xzf criteo.kaggle2014.svm.tar.gz
cd ../
```

## Data Pre-processing
The data is in SvmLight format which is not really appropriate since this dataset is dense. 
Therefore, we perform some pre-processing. 

The entire pre-processing can be run by executing `python $CONDA_PREFIX/pai4sk/local-examples/example-criteo/preprocess-criteo-kaggle.py`

Command to pre-process the data available in specific directory:
`python $CONDA_PREFIX/pai4sk/local-examples/example-criteo/preprocess-criteo-kaggle.py --data_path path/to/data` directory.

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.

## Training and Evaluating a Logistic Regression Model
One can run the entire example by executing `python $CONDA_PREFIX/pai4sk/local-examples/example-criteo/example-criteo-kaggle.py`

Command to run example using the pre-processed data in specific directory.
`python $CONDA_PREFIX/pai4sk/local-examples/example-criteo/example-criteo-kaggle.py --data_path path/to/data` directory.

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.

Command to run example on GPU 0:
python $CONDA_PREFIX/pai4sk/local-examples/example-criteo/example-criteo-kaggle.py --use_gpu

Command to run example on specified GPUs:
python $CONDA_PREFIX/pai4sk/local-examples/example-criteo/example-criteo-kaggle.py --use_gpu --device_ids 1 2

Command to run example with fit_intercept:
python $CONDA_PREFIX/pai4sk/local-examples/example-criteo/example-criteo-kaggle.py --use_gpu --fit_intercept

Command to collect training information:
```
python $CONDA_PREFIX/pai4sk/local-examples/example-criteo/example-criteo-kaggle.py --use_gpu --return_training_history=full

python $CONDA_PREFIX/pai4sk/local-examples/example-criteo/example-criteo-kaggle.py --use_gpu --return_training_history=summary
```
One should see the following output on P9+V100 System (--use_gpu --device_ids 0):

```
Data load time (s): 29.58
[pai4sk] Training time (s):  27.78
[pai4sk] Logarithmic loss:   0.4558
[pai4sk.lmodel] Training time (s):  32.96
[pai4sk.lmodel] Logarithmic loss:   0.4558
[sklearn] Training time (s):  582.58
[sklearn] Logarithmic loss:   0.4558
```

Times will vary depending on the system architecture. 

Note: By default the training will run on CPU. With --use_gpu option, default device_ids list contains only GPU 0 for pai4sk and all the available GPUs for pai4sk.lmodel.
User can set the parameter device_ids to run training on required GPUs.

## Privacy

In this example we additionally provide one example to offer new functionality to train a logistic regression model in such a way that can guarantee a desired level of privacy.

The entire pre-processing can be run by executing `python $CONDA_PREFIX/pai4sk/local-examples/example-criteo/preprocess-criteo-kaggle-privacy.py`

Command to run example:

python $CONDA_PREFIX/pai4sk/local-examples/example-criteo/example-criteo-kaggle-privacy.py

&copy; Copyright IBM Corporation 2018, 2019
