# mpi-examples
This directory contains examples of using the pai4sk through MPI.

Use snaprun to launch examples that use pai4sk through MPI on to a single host or
multiple hosts. snaprun takes care of generating the MPI rank file based on the
hosts details given as arguments to snaprun. All GPUs on every host passed as
argument to snaprun will be used. See sample commands in the
"Running the examples" section below.

## To run examples on multiple hosts

To run pai4sk with MPI on more than one host,

all the hosts should have

```
pai4sk installed and
Passwordless ssh is enabled between hosts

```

To run the examples provided here, as the examples compare pai4sk with scikit-learn,
install `scikit-learn` and `pandas` also on all the hosts.

```
conda install scikit-learn pandas
```

#### Using Ubuntu as the host OS

Spectrum MPI requires the usage of ptrace. By default, Ubuntu does not allow this. So if the host OS is Ubuntu, in order to turn on this capability, the following command should be run on the host system:

```
$ sudo bash -c "echo '0' > /proc/sys/kernel/yama/ptrace_scope"
```

#### snaprun on multiple hosts with multiple interfaces

It is possible in a distributed environment to have more than one network interface for each host. In such a scenario, Spectrum MPI by default, uses any and all interfaces that are "up" to communicate with a host. To avoid problems in such cases you can tell MPI to use given interface. E.g.:

```
$ snaprun --mpiarg "--mca btl_tcp_if_include ib0" ...

$ snaprun --mpiarg "--mca btl_tcp_if_exclude lo,enp1s0f2" ...
```

More details are available on the Spectrum MPI website https://www.ibm.com/support/knowledgecenter/en/SSZTET_10.1/smpi02/smpi02_setting_mca_mpirun.html.

## Running the examples

If pandas and scikit-learn are not already installed in the conda environment,
run `conda install pandas scikit-learn` before running the examples.

Note that `scikit-learn` does not provide an MPI implementation. We will however still perform this comparison in order to mainly verify the accuracy metrics. If you want to skip the scikit-learn part of the examples, you can use --skip_sklearn option on the command line of examples. This is useful specifically when large data is being used by example and in such cases scikit-learn can take a long time to perform training.

Most of the examples take command line options like --data_path, --use_gpu, --device_ids, --skip_sklearn.

By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.

By default the training will run on CPU. With --use_gpu option, default device_ids list contains all GPUs on the given hosts which are passed as arguments to snaprun.

If snaprun tool is used to run these examples, the tool automatically determines the devices available on all the hosts and uses all the devices. In this case, the device_ids parameter should not be set.

If mpirun is directly used instead of snaprun tool, then the device_ids parameter can still be passed to run training on desired GPUs of all hosts.

Please see the individual sub-directories for more details on running each of the examples.

&copy; Copyright IBM Corporation 2018, 2019
