# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2018, 2019. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

from __future__ import print_function
import argparse
import os
import time

defaultPath = os.getcwd()
CLI=argparse.ArgumentParser()
CLI.add_argument(
   "--data_path",
   type=str,
   default=defaultPath
)


args = CLI.parse_args()

from pai4sk.sml_io import load_from_snap_format

# load train data
t0 = time.time()
data = load_from_snap_format(args.data_path + "/data/criteo_tb_1b.snap")
t1 = time.time()
print("Time to load training data (s): %f" % (t1-t0))

# load test data
t0 = time.time()
data_t = load_from_snap_format(args.data_path + "/data/criteo_tb.t.snap")
t1 = time.time()
print("Time to load test data (s): %f" % (t1-t0))

# create model
from pai4sk import LogisticRegression
clf = LogisticRegression(use_gpu=True, regularizer=10, num_threads=64)

# training
t0 = time.time()
hist = clf.fit(data)
t1 = time.time()
print("Training time (s): %f" % (t1-t0))

# inference
proba = clf.predict_proba(data_t)

# evaluation
from pai4sk.sml_metrics import log_loss
loss = log_loss(data_t, proba)
print("Logistic Loss: %f" % (loss))
